//#define VISUALSTUDIO

//if we aren't programming in visual studio
#ifndef VISUALSTUDIO
	//include the me218_c32.h file from the c directory
	#include <ME218_C32.h>
#endif
#ifdef VISUALSTUDIO
	//otherwise, include the me218_c32.h file from the sources directory
	#include "ME218_C32.h"
#endif

#include "DigitalIO.h"

//InputConfig is a module function for modifying direction registers on configurable IO ports.
//Inputs:
//	DirPort	- type C32Register - the pointer to the direction register
//	Port	- type C32Register - the pointer to the data register
//	BitMask	- type C32Register - the configuration for each bit. 1 means that bit will be configured/used as an input
//Output: 
//	Returns type InputT with Input configuration data.
static InputT InputConfig(C32Register *DirPort, C32Register *Port, C32Register BitMask);

//OutputConfig is a module function for modifying direction registers on configurable IO ports.
//Inputs:
//	DirPort	- type C32Register - the pointer to the direction register
//	Port	- type C32Register - the pointer to the data register
//	BitMask	- type C32Register - the configuration for each bit. 1 means that bit will be configured/used as an output
//Output: 
//	Returns type OutputT with Output configuration data.
static OutputT OutputConfig(C32Register *DirPort, C32Register *Port, C32Register BitMask);

InputT CreatePortEInput(C32Register BitMask)
{
	//create variable Input of type InputT
	InputT Input;
	//update the internal memory of Input with pointers for port E.
	Input.DirPort=&PORTE;
	Input.Port=&PORTE;
	//set the bitmask for Input equal to BitMask
	Input.BitMask=BitMask;
	//Return Input
	return Input;
}
InputT CreatePortTInput(C32Register BitMask)
{
	//return the result of InputConfig with pointers for port T and BitMask
	return InputConfig(&DDRT,&PTT,BitMask);
}
InputT CreatePortMInput(C32Register BitMask)
{
	//Return the result of InputConfig with pointers for port M and BitMask
	return InputConfig(&DDRM,&PTM,BitMask);
}
InputT CreatePortADInput(C32Register BitMask)
{
	//create variable Input of type InputT
	InputT Input;
	//update the internal memory of Input with pointers for port AD.
	Input.DirPort=&PTIAD;
	Input.Port=&PTIAD;
	//set the bitmask for Input equal to BitMask
	Input.BitMask=BitMask;
	//Return Input
	return Input;
}
OutputT CreatePortTOutput(C32Register BitMask)
{
	//return the result of OutputConfig with pointers for port T and BitMask
	return OutputConfig(&DDRT,&PTT,BitMask);
}
OutputT CreatePortMOutput(C32Register BitMask)
{
	//return the result of OutputConfig with pointers for port M and BitMask
	return OutputConfig(&DDRM,&PTM,BitMask);
}
OutputT CreatePortADOutput(C32Register BitMask)
{
	//create variable Output of type OutputT
	OutputT Output;
	//update the internal memory of Output with pointers for port AD
	Output.DirPort=&PTAD;
	Output.Port=&PTAD;
	//set output.bitmask equal to BitMask.
	Output.BitMask=BitMask;
	//Return Output
	return Output;
}
InputT InputConfig(C32Register *DirPort, C32Register *Port, C32Register BitMask) 
{
	//create variable Input of type InputT
	InputT Input;
	//set internal variables equal to input variables
	Input.DirPort=DirPort;
	Input.Port=Port;
	Input.BitMask=BitMask;
	//modify the port's direction register with 0's in the bitmasked locations	
	*(Input.DirPort)=(*(Input.DirPort) & (~Input.BitMask));
	//return Input
	return Input;
}
OutputT OutputConfig(C32Register *DirPort, C32Register *Port, C32Register BitMask)
{
	//create variable Output of type OutputT
	OutputT Output;
	//set internal variables equal to input variables
	Output.DirPort=DirPort;
	Output.Port=Port;
	Output.BitMask=BitMask;
	//modify the port's direction register with 1's in the bitmasked locations	
	*(Output.DirPort)=(*(Output.DirPort) | Output.BitMask);
	//return Output
	return Output;
}
C32Register InputRead(InputT Input)
{
	//return the data register bitmasked with Input.BitMask
	return (*(Input.Port) & Input.BitMask);
}
C32Register OutputRead(OutputT Output)
{
	//return the data register bitmasked with Output.BitMask
	return (*(Output.Port) & Output.BitMask);
}
void OutputWrite(OutputT Output, C32Register Value) 
{
	//create temporary variables a and b of type C32Register
	C32Register a,b;
	//set a equal to the bits we want to change
	a=(Output.BitMask & Value);
	//set b equal to the bits we don't want to change
	b=((~Output.BitMask) & *(Output.Port));
	//bitwise-or a and b together and set the data register equal to that value.
	*(Output.Port)=(a | b);
}