#ifndef _IO_H
#define _IO_H

typedef unsigned char C32Register;

typedef struct
{
	//holder for the pointer to the direction register
	C32Register *DirPort;
	//holder for the pointer to the data register
	C32Register *Port;
	//defines which bits are configured as an input
	C32Register BitMask;
} InputT;

typedef struct
{
	//holder for the pointer to the direction register
	C32Register *DirPort;
	//holder for the pointer to the data register
	C32Register *Port;
	//defines which bits are configured as an input	
	C32Register BitMask;
} OutputT;

//The following functions create input variables and possibly configure bits for each port.
//Inputs:
//	BitMask	- type C32Register - the configuration for each bit. 1 means that bit will be configured/used as an input
//Returns type InputT with Input configuration data.
InputT CreatePortEInput(C32Register BitMask);
InputT CreatePortTInput(C32Register BitMask);
InputT CreatePortMInput(C32Register BitMask);
InputT CreatePortADInput(C32Register BitMask);

//The following functions create output variables and possibly configure bits for each port.
//Inputs:
//	BitMask	- type C32Register - the configuration for each bit. 1 means that bit will be configured/used as an output
//Returns type OutputT with Output configuration data.
OutputT CreatePortTOutput(C32Register BitMask);
OutputT CreatePortMOutput(C32Register BitMask);
OutputT CreatePortADOutput(C32Register BitMask);

//InputRead() reads the value of input bits located in the configured Input variable.  InputRead() only returns the value of bits configured in Input, and sets all other bits to zero.
//Inputs:
//	Input - type InputT - holds configuration data
//Returns input data in type C32Register
C32Register InputRead(InputT Input);

//OutputRead() reads the value of output bits located in the configured Output variable.  OutputRead() only returns the value of bits configured in Output, and sets all other bits to zero.
//Inputs:
//	Input - type InputT - holds configuration data
//Returns input data in type C32Register
C32Register OutputRead(OutputT Output);

//OutputWrite() sends data to the output bits configured in the Output variable.
void OutputWrite(OutputT Output, C32Register Value);

#endif