//#define VISUALSTUDIO
#ifndef VISUALSTUDIO
	#include <ME218_C32.h>
#endif
#ifdef VISUALSTUDIO
	#include "ME218_C32.h"
#endif

#include "PrizeLED.h"
#include "ShiftRegister.h"

//Define the default prize pattern
#define DEFAULTPATTERN 3
//define the total number of LEDs in the array.
#define LED_TOTAL 9
//define the positive mask for all 9 LEDs
#define Mask9LEDs 0x1FF
//create module variable Clock, of type OutputT
static OutputT Clock;
//create module variable Data, of type OutputT
static OutputT Data;
//create module variable PrizePattern, of type unsigned int, and initialize to DEFAULTPATTERN
static unsigned int PrizePattern=DEFAULTPATTERN;

//declare internal function LightLEDpattern
static void LightLEDpattern(unsigned int pattern);

void PrizeLEDInit(OutputT PrizeLEDClock, OutputT PrizeLEDData)
{
	//Initialize module variable Clock with PrizeLEDClock
	Clock=PrizeLEDClock;
	//Initialize module variable Data with PrizeLEDData
	Data=PrizeLEDData;
	//Initialize shift register outputs
	InitShiftRegister(Clock,Data);
	//Light initial blank LED pattern.
	LightLEDpattern(0);
}

void LightLEDpattern(unsigned int pattern)
{
	//declare counter variable i, of type int.
	int i;
	//set module variable PrizePattern equal to input variable pattern
	PrizePattern = pattern; 
	//for LED_Total times, 
	for(i=0;i<LED_TOTAL;i++)
	{
		//if pattern and BIT0HI share a bit,
		if(pattern & BIT0HI)
		{
			//light the next LED
			LightNext(Clock,Data);
		}
		//otherwise
		else
		{
			//don't light the next LED
			DontLightNext(Clock,Data);
		}
		//shift pattern to the right and repeat for loop
		pattern >>= 1;
	}
}

void PrizeLEDReset(void)
{
	//light the default LED pattern
	LightLEDpattern(DEFAULTPATTERN);
}

void PrizeLEDShiftLeft(void)
{ 
	//declare prizePos, of type int, and initialize to the current prizepattern masked with the 9 LED bitmask
	int prizePos=PrizePattern & Mask9LEDs;
	//declare prizePosNew, and initialize to prizepos shifted to the left
	int prizePosNew=prizePos << 1;
	//if prizepos's 8th bit is 1
	if ((prizePos & BIT8HI) !=0) 
	{
		//then set prizeposnew's 0th bit to 1.
		prizePosNew |= BIT0HI;
	}
	//and light LEDs with new pattern
	LightLEDpattern((PrizePattern & (~Mask9LEDs)) | prizePosNew);
}

void PrizeLEDShiftRight(void)
{ 
	//declare prizePos, of type int, and initialize to the current prizepattern masked with the 9 LED bitmask
	int prizePos=PrizePattern & Mask9LEDs;
	//declare prizePosNew, and initialize to prizepos shifted to the left
	int prizePosNew=prizePos >> 1;
	//if prizepos's 0th bit is 1
	if ((prizePos & BIT0HI) !=0) 
	{
		//then set prizeposnew's 8th bit to 1.
		prizePosNew |= BIT8HI;
	}
	//and light LEDs with new pattern
	LightLEDpattern((PrizePattern & (~Mask9LEDs)) | prizePosNew);
}

unsigned int PrizeLEDGetPattern(void){
	//return the internal module variable PrizePattern
	return PrizePattern;
}