//#define VISUALSTUDIO
#ifndef VISUALSTUDIO
	#include <ME218_C32.h>
#endif
#ifdef VISUALSTUDIO
	#include "ME218_C32.h"
#endif

//define scaling for potentiometer
#define SCALE 45
//define deadband for potentiometer
#define DEADBAND 5
//define wait time for reset function
#define RESETWAIT 10

#include "ADS12.h"
#include "CustomTime.h"
#include "Stepper.h"
#include "DigitalIO.h"

//declare module variable RightLimit of type InputT
static InputT RightLimit;
//declare module variable LeftLimit of type InputT
static InputT LeftLimit;
//declare module variable Stepper of type OutputT
static OutputT Stepper;
//declare module function left()
static void Left(void);
//declare module function right()
static void Right(void);
//declare module variable position, of type short int
static int position=0;

void StepperInit(InputT rl, InputT ll, OutputT stpr)
{
	//initialize module variable RightLimit to rl.
	RightLimit = rl;
	//initialize module variable LeftLimit to ll.
	LeftLimit = ll;
	//initialize module variable stepper to stpr.
	Stepper = stpr;
}

void Left(void)
{
	//depending on the current state of the Stepper output,
	switch (OutputRead(Stepper))
	{
		//if stepper is in state 0x30, send it to state 0x60
		case 0x30: OutputWrite(Stepper, 0x60); break;
		//if stepper is in state 0x60, send it to state 0xC0
		case 0x60: OutputWrite(Stepper, 0xC0); break;
		//if stepper is in state 0xC0, send it to state 0x90
		case 0xC0: OutputWrite(Stepper, 0x90); break;
		//if stepper is in state 0x90, send it to state 0x30
		case 0x90: OutputWrite(Stepper, 0x30); break;
		//otherwise, send it to state 0x30.
		default: OutputWrite(Stepper, 0x30); break; 
	}
	//increment internal position
	position++; 
}

void Right(void)
{
	//depending on the current state of the Stepper output,
	switch (OutputRead(Stepper))
	{
		//if stepper is in state 0x30, send it to state 0x90
		case 0x30: OutputWrite(Stepper, 0x90); break;
		//if stepper is in state 0x90, send it to state 0xC0
		case 0x90: OutputWrite(Stepper, 0xC0); break;
		//if stepper is in state 0xC0, send it to state 0x60
		case 0xC0: OutputWrite(Stepper, 0x60); break;
		//if stepper is in state 0x60, send it to state 0x30
		case 0x60: OutputWrite(Stepper, 0x30); break;
		//otherwise, send it to state 0x30.
		default: OutputWrite(Stepper, 0x30); break; 
	}
	//decrement internal position
	position--; 
}

void StepperTrack(void)
{
	//declare voltage, of type short, and initialize to ADS12_ReadADPin(0).
	short voltage=ADS12_ReadADPin(0);
	//scale voltage
	voltage=(short)(((long)voltage * SCALE)/255);
	//if the position is less than the scaled voltage minus the deadband
	if (position<(voltage-DEADBAND)) 
	{
		//and if the left limit switch is not depressed
		if (InputRead(LeftLimit)==0) 
		{
			//step left
			Left();
		}
		//otherwise, limit switch is depressed
		else 
		{
			//reset position to experimentally-determined left position
			position=178;
		}
	}
	//if the position is greater than the scaled voltage plus the deadband
	if (position>(voltage+DEADBAND)) 
	{
		//and if the right limit switch is not depressed
		if (InputRead(RightLimit)==0) 
		{
			//step right
			Right();
		}
		else 
		{
			//reset position to 0
			position=0;
		}
	}
}

void StepperReset(void)
{	
	//while the rightlimit switch is not depressed
	while (InputRead(RightLimit)==0)
	{
		//step right
		Right();
		//Wait RESETWAIT*2 milliseconds.
		Wait(RESETWAIT);
	}
	//reset position to zero.
	position=0;
} 